// ============================================================================
// ==                                                                        ==
// == AMDiS - Adaptive multidimensional simulations                          ==
// ==                                                                        ==
// ==  http://www.amdis-fem.org                                              ==
// ==                                                                        ==
// ============================================================================
//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.



/** \file ParallelProblemStatBase.h */

#ifndef AMDIS_PARALLEL_PROBLEM_STAT_BASE_H
#define AMDIS_PARALLEL_PROBLEM_STAT_BASE_H

#include "parallel/MeshDistributor.h"
#include "ProblemVec.h"

namespace AMDiS {

  class ParallelProblemStatBase : public ProblemVec
  {
  public:
    ParallelProblemStatBase(std::string nameStr,
			    ProblemIterationInterface *problemIteration = NULL)
      : ProblemVec(nameStr, problemIteration),
	meshDistributor(NULL)
    {}

    virtual ~ParallelProblemStatBase() {}

    void buildAfterCoarsen(AdaptInfo *adaptInfo, Flag flag,
			   bool assembleMatrix,
			   bool assembleVector);

    virtual void addToMeshDistributor(MeshDistributor& m);

  protected:
    void processMemUsage(double& vm_usage, double& resident_set);

  protected:
    MeshDistributor *meshDistributor;     
  };
}

#endif
